/*globals CodeMirror: false, CoB:false  */
(function($) {
    $.fn.advanceEditor = function(opts) {

        if (this.length > 1) {
            window.console.log("Can't apply this to an array of elements");
        }

        var _input = this[0];

        var _editor,
            _currentLine,
            _container,
            _$lineCursorPositionInput,
            _$columnCursorPositionInput,
            _$errorTitle,
            _$errorMsg;

        var defaultOpts = {
            // cob specific properties
            verticalResize: false,
            minHeight: 300,
            // end

            smartIndent: false,
            tabSize: 4,
            indentUnit: 4,
            indentWithTabs: true,
            lineNumbers: true,
            lineWrapping: true,
            matchBrackets: true,
            onCursorActivity: function() {
                _editor.setLineClass(_currentLine, null);
                _currentLine = _editor.setLineClass(_editor.getCursor().line, "activeline");

                if (_$lineCursorPositionInput) {
                    _$lineCursorPositionInput.val(_editor.getCursor().line + 1);
                }

                if (_$columnCursorPositionInput) {
                    var lineContent = _editor.getLine(_editor.getCursor().line).substring(0, _editor.getCursor().ch);
                    var numberOfTabs = (lineContent.split(/\t/g).length - 1);
                    _$columnCursorPositionInput.val((lineContent.length - numberOfTabs) + (numberOfTabs * this.tabSize));
                }
            }
        };

        var options = $.extend(defaultOpts, opts || {});

        var _enableVerticalResize = function(container, options) {
            var resizeDiv = $("<div></div>").addClass("CodeMirror-vresize");
            container.append(resizeDiv);

            var wrapper = container.find(".CodeMirror-wrap");

            resizeDiv.bind("mousedown", function(event) {
                event.preventDefault();

                var resizableElement = wrapper;
                resizableElement._startY = event.pageY;
                resizableElement._startHeight = parseInt(wrapper.height(), 10);

                $(document).bind("mousemove", function(event) {
                    event.preventDefault();

                    var newHeight = resizableElement._startHeight + (event.pageY - resizableElement._startY);
                    if (newHeight >= options.minHeight) {
                        resizableElement.css("height", newHeight + "px");
                        resizableElement.find(".CodeMirror-scroll").css("height", newHeight + "px");
                        resizableElement.find(".CodeMirror-gutter").css("height", newHeight + "px");
                    }

                });

                $(document).bind("mouseup", function(event) {
                    $(document).unbind("mousemove");
                    $(document).unbind("mouseup");
                });
            });
        };

        var _addBottomToolbarInfo = function(container) {
            var htmlMarkup = [
                '<div class="CodeMirror-bottomtoolbar form-inline row-fluid">',
                '<div class="span6">',
                '<div class="js-error-msg centered no-margin alert alert-error hidden">',
                '<strong class="js-error-title" rel="localize[error.warning]">WARNING: </strong>',
                '</div>',
                '</div>',
                '<div class="span6 align-right">',
                '<label>L:</label>',
                '<input type="text" class="input-mini CodeMirror-line" disabled=disabled>',
                '<label>C:</label>',
                '<input type="text" class="CodeMirror-column input-mini" disabled=disabled>',
                '</div>',
                '</div>'
            ].join(' ');

            var $bottomToolbar = $(htmlMarkup);
            _$errorTitle = $(".js-error-title", $bottomToolbar);
            _$errorMsg = $(".js-error-msg", $bottomToolbar);
            _$lineCursorPositionInput = $bottomToolbar.find(".CodeMirror-line");
            _$columnCursorPositionInput = $bottomToolbar.find(".CodeMirror-column");

            container.append($bottomToolbar);
        };

        var init = function() {
            _editor = CodeMirror.fromTextArea(_input, options);
            _currentLine = _editor.setLineClass(0, "activeline");
            _container = $(_input).parent();

            _addBottomToolbarInfo(_container);

            if (options.verticalResize) {
                _enableVerticalResize(_container, options);
            }


        };

        init();

        return {
            getText: function() {
                return _editor.getValue();
            },
            setText: function(text) {
                if (text) {
                    _editor.setValue(text);
                } else {
                    _editor.setValue("");
                }
                _editor.clearHistory();
            },
            getSelectedText: function() {
                if (_editor.somethingSelected()) {
                    return _editor.getSelection();

                } else {
                    return undefined;
                }
            },
            getSearchCursor: function(text, pos, caseFold) {
                return _editor.getSearchCursor(text, pos, caseFold);
            },
            getCursor: function(text) {
                return _editor.getCursor();
            },
            setSelection: function(from, to) {
                _editor.setSelection(from, to);
            },
            markText: function(from, to, op) {
                return _editor.markText(from, to, op);
            },
            getSelection: function() {
                return _editor.getSelection();
            },
            setCursor: function(pos) {
                return _editor.setCursor(pos);
            },
            historySize: function() {
                return _editor.historySize();
            },
            showTabs: function(show) {
                if (show) {
                    $("<style id='tabsCss' type='text/css'> " +
                      ".cm-tab:after{" +
                      "position: relative; overflow: visible; left: -1.4em; " +
                      "display: -moz-inline-block; display: -webkit-inline-block; display: inline-block; " +
                      "width: 0px; " +
                      "color: # aaa; " +
                      "content: '\\21e5';" +
                      "} " +
                      "</style>")
                        .appendTo('#'+opts.cobAppInstanceId);
                } else {
                    $("#tabsCss").remove();
                }
            },
            showErrorMsg: function(msg) {
                _$errorMsg.html("");
                _$errorMsg.append(_$errorTitle);
                _$errorMsg.append(msg);

                _$errorMsg.removeClass("hidden");
            },
            hideErrorMsg: function() {
                _$errorMsg.addClass("hidden");
            }
        };
    };
}(jQuery));
